﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;

public class Cuffs : MVRScript {

    #region PluginInfo    
    public string pluginAuthor = "Hazmhox and Frief";
    public string pluginName = "Shackels";
    public string pluginVersion = "1";
    public string pluginDate = "26/05" + "/2022";
    public string pluginDescription = @"Used to open and close the cuffs";
    #endregion

    public GameObject player;
    public GameObject KS_Shed;

    protected JSONStorableFloat ScrubA;
    protected JSONStorableFloat ScrubB;
    protected JSONStorableFloat ScrubDistance;


    protected JSONStorableFloat yRotation;
    protected JSONStorableFloat yRotation2;

    private float previousValue1;
    private float previousValue2;


    private float previousZValue;
    private float currentZValue;

    public Transform ObjectRoot;
    public Transform ObjectRoot2;
    public Transform Cuff_R;
    public Transform Cuff_L;
    public Transform Cuff_Open_L;
    public Transform Cuff_Open_R;
    public float distance;
    public float max_distance = 0.03216213f;



    private bool editorInitialized = false;
    private int initLoops = 0;
    private int maxInitLoops = 1500; // 3600 for one minute

    public override void Init()
    {
        try
        {
            // make atom selector
            atomJSON = new JSONStorableStringChooser("connectedToAtom", SuperController.singleton.GetAtomUIDs(), null, "Connect to Atom", SyncAtom);
            RegisterStringChooser(atomJSON);
            SyncAtomChocies();
            UIDynamicPopup dp = CreateScrollablePopup(atomJSON);
            dp.popupPanelHeight = 1100f;
            // want to always resync the atom choices on opening popup since atoms can be added/removed
            dp.popup.onOpenPopupHandlers += SyncAtomChocies;

            

            if (containingAtom.type != "CustomUnityAsset")
            {
                SuperController.LogError("Cuffs - Please add this script on a Custom Unity Asset ( script disabled).");
                return;
            }
            //**************  max_distance
       //     ScrubDistance = new JSONStorableFloat("Open/Close Left", 0f, OnSliderChanged1, 0.03f, 180f);
         //   RegisterFloat(ScrubDistance);
           // CreateSlider(ScrubDistance, false);


            //**************  ScrubA
            ScrubA = new JSONStorableFloat("Open/Close Left", 0f, OnSliderChanged1, 0f, 180f);
            RegisterFloat(ScrubA);
            CreateSlider(ScrubA, false);


            //**************  ScrubB
            ScrubB = new JSONStorableFloat("Open/Close Right", 0f, OnSliderChanged2, -180f, 0f);
            RegisterFloat(ScrubB);
            CreateSlider(ScrubB, false);

            // A tiny bit of help to give some info to your fellow creator and/or player
            JSONStorableString helpText = new JSONStorableString("About",
                "<color=#000><size=35><b>About</b></size></color>\n\n" +
                "<color=#333>" +
                "Frief CuffDoors Plugin , with help of Hazmhox.\n\n" +
                "The Cuffs can be triggered using Acidbubbles Timeline to open and close.\n\n" +
                "if you have a Cuff open and save the scene the position is also saved.\n\n" +
                "</color>");
            UIDynamic helpTextfield = CreateTextField(helpText, true);
            helpTextfield.height = 1100.0f;

        }
        catch (Exception e)
        {
            SuperController.LogError("Exception caught: " + e);
        }
    }

    private FreeControllerV3 ChainEnd;
    // Use this for initialization
    void Start()
    {

    }

    // Update is called once per frame
    void Update()
    {
        if (connectedBody == null)
        {
            connectTo(findChainEnd(connectedToAtom));
        }

        else
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("Cuff: This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }
        }

       // distance = Vector3.Distance(ObjectRoot.position, ObjectRoot2.position);
        // SuperController.LogMessage("distance: "+ distance);

        // if (distance > max_distance)
        //{
          //  SuperController.LogMessage("to far ");
            //Get the unit vector between two objects
            //Vector3 pos = (ObjectRoot.position - ObjectRoot2.position).normalized;
           
            //The coordinates of object A plus the distance vector
            //ObjectRoot2.position = ( ObjectRoot.position+ ObjectRoot2.position)* distance+ObjectRoot2.position;

        //}

    }

    private bool InitEditor()
    {
        if (editorInitialized == true) return true;

        ObjectRoot = getObjectRoot(containingAtom.reParentObject, "control");
        if (ObjectRoot == null)
        {
            SuperController.LogError("ObjectRoot could not be found");
            return false;
        }


        ObjectRoot2 = getObjectRoot(connectedToAtom.reParentObject, "control");
        
        if (ObjectRoot2 == null)
        {
            SuperController.LogError("ObjectRoot2 could not be found...");
            return false;
        }


        Cuff_Open_L = ObjectRoot.Find("Base/Cuff/Open");

        if (Cuff_Open_L == null)
        {
            SuperController.LogError("Cuff_Open_L could not be found");
            return false;
        }

        Cuff_Open_R = ObjectRoot2.Find("Base/Cuff/Open");

        if (Cuff_Open_R == null)
        {
            SuperController.LogError("Cuff_Open_R");
            return false;
        }

        Cuff_R = ObjectRoot.Find("Base/Cuff");
        Cuff_L = ObjectRoot.Find("Base/Cuff");



        // I found my transforms, now the rest
        if (Cuff_Open_L)
        {
            editorInitialized = true;

            OnSliderChanged1(0f);
            OnSliderChanged2(0f);

            return true;
        }
        else
        {
            SuperController.LogError("Cuff : your CUA is missing mandatory components.");
            initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
            return false;
        }

        return false;
    }

    // Recursive function searching for a transform containing the string set in objectNameSearchString
    // It is used the first time for the initialization of the script
    // This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
    private Transform getObjectRoot(Transform parent, string rootName)
    {
        foreach (Transform child in parent)
        {
            if (child.name.Contains(rootName))
            {
                return child;
            }
            else
            {
                Transform childSearch = getObjectRoot(child, rootName);
                if (childSearch != null) return childSearch;
            }
        }

        return null;
    }

    private Transform getChildRoot(Transform parent, string searchName)
    {
        foreach (Transform child in parent)
        {
            SuperController.LogMessage(">>" + child.name);
            if (child.name == searchName)
            {

                return child;
            }
            else
            {
                Transform childSearch = getChildRoot(child, searchName);
                if (childSearch != null) return childSearch;
            }
        }

        return null;
    }

    void OnSliderChanged1(float rotation)
    {

        {
            if (Cuff_Open_L == null) return;

            var Slider1 = ScrubA.val;
            var a = Slider1 - previousValue1;
            Cuff_Open_L.localRotation = Quaternion.Euler(0f, 0f, Slider1);
            previousValue1 = ScrubA.val;
        }

    }

    void OnSliderChanged2(float rotation)
    {

        {
            if (Cuff_Open_R == null) return;

            var Slider2 = ScrubB.val;
            var b = Slider2 - previousValue1;
            Cuff_Open_R.localRotation = Quaternion.Euler(0f, 0f, Slider2);
            previousValue2 = ScrubB.val;
        }

    }

    private IEnumerator WaitFor(Func<bool> test, float timeoutSeconds = 3.0f, float delay = 0.2f)
    {
        float waited = 0;
        while (!test() && waited <= timeoutSeconds)
        {
            waited += delay;
            yield return new WaitForSeconds(delay);
        }
    }



Transform GetChainEnd( Transform crnt)
    {
        Transform FoundTrans = null;
        //SuperController.LogMessage(crnt.name);
        if (crnt.name == "ChainEnd")
            return crnt;

        for (int i = 0; i < crnt.childCount; i++)
        {
            FoundTrans = GetChainEnd(crnt.GetChild(i));
            if (FoundTrans != null)
			{
                return FoundTrans;
			}
        }

        return FoundTrans;
    }

    protected JSONStorableStringChooser atomJSON;
    protected void SyncAtomChocies()
    {
        List<string> atomChoices = new List<string>();
        atomChoices.Add("None");
        foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
        {
            if ( findChainEnd(SuperController.singleton.GetAtomByUid(atomUID)) != null )
                atomChoices.Add(atomUID);
        }
        atomJSON.choices = atomChoices;
    }

    protected Atom connectedToAtom = null;
    protected void SyncAtom(string atomUID)
    {
        if (atomUID != null)
        {
            connectedToAtom = SuperController.singleton.GetAtomByUid(atomUID);
            connectTo(findChainEnd(connectedToAtom));
        }
    }

    private Rigidbody connectedBody = null;

    Rigidbody findChainEnd(Atom connectedAtom )
    {
        if ( connectedAtom == null )
            return null;

        foreach ( Rigidbody rb in connectedAtom.GetComponentsInChildren<Rigidbody>() )
        {
            if (rb.name == "ChainEnd")
                return rb;
        }

        return null;
    }

    void connectTo( Rigidbody toConnect )
    {
        if (connectedToAtom == null || toConnect == null)
            return;

        foreach (Rigidbody rb in containingAtom.rigidbodies)
        {
            if (rb.name == "object")
            {
                ConfigurableJoint fj = rb.gameObject.GetComponent<ConfigurableJoint>();
                if (fj != null)
                {
                    fj.connectedBody = toConnect;
                    connectedBody = toConnect;
                    fj.xMotion = ConfigurableJointMotion.Locked;
                    fj.yMotion = ConfigurableJointMotion.Locked;
                    fj.zMotion = ConfigurableJointMotion.Locked;
                    fj.autoConfigureConnectedAnchor = false;
                    fj.connectedAnchor = new Vector3(0.0f, -0.05f, 0.0f);
                    rb.mass = toConnect.mass;
                }
                else if (fj == null)
                {
                    fj = rb.gameObject.AddComponent<ConfigurableJoint>();
                    fj.connectedBody = toConnect;
                    connectedBody = toConnect;
                    fj.xMotion = ConfigurableJointMotion.Locked;
                    fj.yMotion = ConfigurableJointMotion.Locked;
                    fj.zMotion = ConfigurableJointMotion.Locked;
                    fj.autoConfigureConnectedAnchor = false;
                    fj.connectedAnchor = new Vector3(0.0f, -0.05f, 0.0f);
                    rb.mass = toConnect.mass;
                }
                else
                {
                    SuperController.LogMessage("Hm.. Strange...");
                }
            }
        }
    }


    protected UIDynamicSlider BumbnessSlider;
    protected UIDynamicSlider SmoothnessSlider;
    
    protected void BumbnessCallback(float val)
    {
        
    }

   

}
